@@ -107,11 +107,12 @@ blockRedraws <- function(expr, playState = playDevCur()) {
 	#playState$win$setGeometryHints(da, list(max.width=myW, min.width=myW, 
 	#	max.height=myH, min.height=myH))
 	da$window$freezeUpdates() # hmm
-	try(eval.parent(substitute(expr)))
+	foo <- try(eval.parent(substitute(expr)))
 	da$window$thawUpdates()
 	#playState$win$setGeometryHints(da, list())
 	da$setSizeRequest(-1, -1)
 	playState$skip.redraws <- oval
+	foo
 }
 
 playPrompt <- function(playState, text=NULL) {
@@ -125,7 +126,7 @@ playPrompt <- function(playState, text=NULL) {
 				promptBox$hide()
 			}
 			playThawGUI(playState)
-			return()
+			return(invisible())
 		}
 		# show the prompt widget
 		promptBox["sensitive"] <- TRUE
@@ -138,6 +139,7 @@ playPrompt <- function(playState, text=NULL) {
 		promptLabel$setMarkup(paste("<big><b>", 
 			toString(text), "</b></big>"))
 	})
+	invisible()
 }
 
 rawXLim <- function(playState, space="plot") {
@@ -264,6 +266,8 @@ playDo <- function(playState, expr, space="plot", clip.off=FALSE) {
 			myVp <- trellis.vpname("panel", myCol, myRow, clip.off=clip.off)
 			depth <- downViewport(myVp)
 			on.exit(upViewport(depth), add=TRUE)
+			# TODO: should focus panel or just go to viewport?
+			# (if focus, will destroy any previous focus)
 			#trellis.focus("panel", myCol, myRow, highlight=FALSE)
 			#on.exit(trellis.unfocus())
 		}
@@ -296,8 +300,8 @@ playSelectData <- function(playState, prompt="Click or drag to select data point
 	which <- NULL
 	pos <- NULL
 	if (foo$is.click) {
-		x <- mean(coords$x)
-		y <- mean(coords$y)
+		x <- coords$x[1]
+		y <- coords$y[1]
 		ppxy <- playDo(playState, list(
 				lx=convertX(unit(x, "native"), "points", TRUE),
 				ly=convertY(unit(y, "native"), "points", TRUE),
@@ -320,9 +324,9 @@ playSelectData <- function(playState, prompt="Click or drag to select data point
 			& (min(coords$y) <= data$y) & (data$y <= max(coords$y)))
 		which <- which(ok)
 	}
-	list(which=which, space=foo$space, 
-		x=data$x[which], y=data$y[which], 
-		pos=pos, is.click=foo$is.click)
+	c(list(which=which, x=data$x[which], y=data$y[which], 
+		pos=pos, is.click=foo$is.click),
+		foo)
 }
 
 playPointInput <- function(playState, prompt="Click on the plot") {
@@ -334,6 +338,12 @@ playPointInput <- function(playState, prompt="Click on the plot") {
 	if (!is.null(cur.vp)) on.exit(downViewport(cur.vp), add=TRUE)
 	dc <- grid.locator()
 	if (is.null(dc)) return(NULL)
+	# check for modifier keys
+	ptrInfo <- playState$widgets$drawingArea$window$getPointer()
+	modifiers <- as.flag(0)
+	if (!is.null(ptrInfo$retval))
+		modifiers <- as.flag(ptrInfo$mask)
+	# convert coordinates
 	ndc <- list(x=convertX(dc$x, "npc"), y=convertY(dc$y, "npc"))
 	dc <- lapply(dc, as.numeric)
 	ndc <- lapply(ndc, as.numeric)
@@ -342,7 +352,7 @@ playPointInput <- function(playState, prompt="Click on the plot") {
 	if (space != "page") {
 		coords <- deviceCoordsToSpace(playState, dc$x, dc$y, space=space)
 	}
-	list(coords=coords, space=space, dc=dc, ndc=ndc)
+	list(coords=coords, space=space, dc=dc, ndc=ndc, modifiers=modifiers)
 }
 
 playPolyInput <- function(playState, prompt="Click points to define a region; right-click to end") {
@@ -488,18 +498,20 @@ handleClickOrDrag <- function(da, x0, y0, shape=c("rect", "line"),
 	}
 	gSignalHandlerDisconnect(da, tmpSigR)
 	gSignalHandlerDisconnect(da, tmpSigE)
+	# check for modifier keys
+	ptrInfo <- da$window$getPointer()
+	modifiers <- as.flag(0)
+	if (!is.null(ptrInfo$retval))
+		modifiers <- as.flag(ptrInfo$mask)
+	# clean up
 	da$window$invalidateRect(invalidate.children=FALSE)
 	if (!exists("px1", inherits=FALSE)) return(NULL)
 	dc <- list(x=c(px0$x, px1$x), y=c(px0$y, px1$y))
-	# was it a click or a drag? (threshold 3 pixels) - TODO: better to use timing
+	# was it a click or a drag? (threshold 3 pixels)
+	# TODO: better to use timing
 	is.click <- ((abs(diff(dc$x)) <= 3) && (abs(diff(dc$y)) <= 3))
-	if (is.click) {
-		# coords are +/- 10 pixels
-		dc$x <- dc$x[1] + c(-10, 10)
-		dc$y <- dc$y[1] + c(-10, 10)
-	}
 	ndc <- list(x=dc$x / da.w, y=(da.h - dc$y) / da.h)
-	list(dc=dc, ndc=ndc, is.click=is.click)
+	list(dc=dc, ndc=ndc, is.click=is.click, modifiers=modifiers)
 }
 
 xyCoords <- function(playState, space="plot") {
@@ -511,7 +523,7 @@ xyCoords <- function(playState, space="plot") {
 
 xyData <- function(playState, space="plot") {
 	if (!is.null(playState$data.points)) {
-		return(xy.coords_with_class(playState$data.points))
+		return(xy.coords_with_class(playState, playState$data.points))
 	}
 	if (playState$is.lattice) {
 		if (space == "page") {
@@ -544,11 +556,11 @@ xyData <- function(playState, space="plot") {
 	}
 	x <- callArg(playState, 1)
 	y <- callArg(playState, name="y")
-	xy.coords_with_class(x, y)
+	xy.coords_with_class(playState, x, y)
 }
 
 # adapted from grDevices::xy.coords
-xy.coords_with_class <- function(x, y=NULL, recycle=TRUE) {
+xy.coords_with_class <- function(playState, x, y=NULL, recycle=TRUE) {
 	if (is.null(y)) {
 		if (is.language(x)) {
 			if (inherits(x, "formula") && length(x) == 3) {
